@@ -540,6 +540,10 @@ ioctl_simplestruct_in_execute(const ioctl_tree * node, unsigned long id, void *a
  *
  ***********************************/
 
+/* Note, we don't currently have any of those */
+
+#if 0
+
 static void
 ioctl_varlenstruct_init_from_bin(ioctl_tree * node, const void *data)
 {
@@ -603,6 +607,8 @@ ioctl_varlenstruct_in_execute(const ioctl_tree * node, unsigned long id, void *a
     return 0;
 }
 
+#endif
+
 /***********************************
  *
  * USBDEVFS_REAPURB
