@@ -120,12 +120,14 @@ pthread_mutex_t trap_path_lock = PTHREAD_MUTEX_INITIALIZER;
 #define TRAP_PATH_LOCK pthread_mutex_lock (&trap_path_lock)
 #define TRAP_PATH_UNLOCK pthread_mutex_unlock (&trap_path_lock)
 
+static size_t trap_path_prefix_len = 0;
+
 static const char *
 trap_path(const char *path)
 {
     static char buf[PATH_MAX * 2];
     const char *prefix;
-    size_t path_len, prefix_len;
+    size_t path_len;
     int check_exist = 0;
 
     /* do we need to trap this path? */
@@ -142,19 +144,19 @@ trap_path(const char *path)
 	return path;
 
     path_len = strlen(path);
-    prefix_len = strlen(prefix);
-    if (path_len + prefix_len >= sizeof(buf)) {
+    trap_path_prefix_len = strlen(prefix);
+    if (path_len + trap_path_prefix_len >= sizeof(buf)) {
 	errno = ENAMETOOLONG;
 	return NULL;
     }
 
     /* test bed disabled? */
     strcpy(buf, prefix);
-    strcpy(buf + prefix_len, "/disabled");
+    strcpy(buf + trap_path_prefix_len, "/disabled");
     if (path_exists(buf) == 0)
 	return path;
 
-    strcpy(buf + prefix_len, path);
+    strcpy(buf + trap_path_prefix_len, path);
 
     if (check_exist && path_exists(buf) < 0)
 	return path;
@@ -195,6 +197,30 @@ get_rdev(const char *nodename)
     return makedev(major, minor);
 }
 
+static dev_t
+parse_dev_t(const char *value, const char *source, int error)
+{
+	unsigned long minor, major;
+	char *endptr;
+	major = strtoul(value, &endptr, 10);
+	if (endptr[0] != ':') {
+	    if (error) {
+		fprintf(stderr, "umockdev: $%s (%s) contains no ':'\n", source, value);
+		abort();
+	    } else
+		return (dev_t) -1;
+	}
+	minor = strtoul(endptr + 1, &endptr, 10);
+	if (endptr[0] != '\0') {
+	    if (error) {
+		fprintf(stderr, "umockdev: %s (%s) has invalid minor\n", source, value);
+		abort();
+	    } else
+		return (dev_t) -1;
+	}
+	return makedev(major, minor);
+}
+
 static int
 is_emulated_device(const char *path, const mode_t st_mode)
 {
@@ -393,9 +419,9 @@ ioctl_record_open(int fd)
     if (record_rdev == (dev_t) - 1) {
 	const char *dev = getenv("UMOCKDEV_IOCTL_RECORD_DEV");
 
-	if (dev != NULL) {
-	    record_rdev = (dev_t) atoi(dev);
-	} else {
+	if (dev != NULL)
+	    record_rdev = parse_dev_t(dev, "$UMOCKDEV_IOCTL_RECORD_DEV", 1);
+	else {
 	    /* not recording */
 	    record_rdev = 0;
 	}
@@ -660,10 +686,10 @@ struct script_record_info {
 static void
 init_script_dev_logfile_map(void)
 {
-    int i, dev;
+    int i;
+    dev_t dev;
     char varname[100];
     const char *devname, *logname, *format;
-    char *endptr;
 
     script_dev_logfile_map_inited = 1;
 
@@ -684,10 +710,9 @@ init_script_dev_logfile_map(void)
 	    fprintf(stderr, "umockdev: $%s not set\n", varname);
 	    exit(1);
 	}
-	dev = strtol(devname, &endptr, 10);
-	if (dev != 0 && *endptr == '\0') {
-	    /* if it's a number, then it is an rdev of a device */
-	    /* ...and we should record its path */
+	dev = parse_dev_t(devname, NULL, 0);
+	if (dev != (dev_t) -1) {
+	    /* if it's a dev_t, we should record its path */
 	    const char *devpath;
 	    snprintf(varname, sizeof(varname), "UMOCKDEV_SCRIPT_RECORD_DEVICE_PATH_%i", i);
 	    devpath = getenv(varname);
@@ -1169,6 +1194,49 @@ inotify_add_watch(int fd, const char *path, uint32_t mask)
     return r;
 }
 
+ssize_t readlinkat(int dirfd, const char *pathname, char *buf, size_t bufsiz)
+{
+    const char *p;
+    libc_func(readlinkat, size_t, int, const char*, char*, size_t);
+    size_t r;
+
+    TRAP_PATH_LOCK;
+    p = trap_path(pathname);
+    DBG(DBG_PATH, "testbed wrapped readlinkat (%s) -> %s\n", pathname, p);
+    if (p == NULL)
+	r = -1;
+    else
+	r = _readlinkat(dirfd, p, buf, bufsiz);
+    TRAP_PATH_UNLOCK;
+    return r;
+}
+
+char *canonicalize_file_name(const char *path)
+{
+    const char *p;
+    libc_func(canonicalize_file_name, char*, const char*);
+    char *r, *oldr;
+
+    TRAP_PATH_LOCK;
+    p = trap_path(path);
+    if (p == NULL)
+	r = NULL;
+    else {
+	r = _canonicalize_file_name(p);
+        if (p == path) {
+	    DBG(DBG_PATH, "testbed wrapped canonicalize_file_name(%s) -> %s (not wrapped)\n", path, r);
+	} else if (r != NULL) {
+	    /* cut off the prefix again */
+	    oldr = r;
+	    r = strdup(r + trap_path_prefix_len);
+	    free (oldr);
+	    DBG(DBG_PATH, "testbed wrapped canonicalize_file_name(%s -> %s) -> %s (wrapped)\n", path, p, r);
+	}
+    }
+    TRAP_PATH_UNLOCK;
+    return r;
+}
+
 ssize_t
 read(int fd, void *buf, size_t count)
 {
