@@ -202,6 +202,7 @@ uevent_sender_send(uevent_sender * sender, const char *devpath, const char *acti
     const char *devtype;
     struct udev_device *device;
     struct udev_monitor_netlink_header nlh;
+    static unsigned long long seqnum = 1;
 
     device = udev_device_new_from_syspath(sender->udev, devpath);
     if (device == NULL) {
@@ -223,11 +224,18 @@ uevent_sender_send(uevent_sender * sender, const char *devpath, const char *acti
     strcpy(props + count, "SUBSYSTEM=");
     strcat(props + count, subsystem);
     count += strlen(props + count) + 1;
+    snprintf(props + count, sizeof(props) - count, "SEQNUM=%llu", seqnum++);
+    count += strlen(props + count) + 1;
     if (udev_device_get_devnode(device)) {
         strcpy(props + count, "DEVNAME=");
         strcat(props + count, udev_device_get_devnode(device));
         count += strlen(props + count) + 1;
     }
+    if (udev_device_get_devtype(device)) {
+        strcpy(props + count, "DEVTYPE=");
+        strcat(props + count, udev_device_get_devtype(device));
+        count += strlen(props + count) + 1;
+    }
 
     /* add versioned header */
     memset(&nlh, 0x00, sizeof(struct udev_monitor_netlink_header));
